/*
 * SwerveIO - A versatile open-source FRC swerve drive library.
 *
 * Copyright (C) 2019-2022 Jordan Bancino <jordan@bancino.net>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published with this
 * source code. You should have received a copy of the GNU General Public
 * License with this program. If you did not, contact the distributors of
 * this program and view it here: https://bancino.net/licenses/gpl-3.0.txt
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 */
package net.bancino.robotics.swerveio;

import edu.wpi.first.wpilibj.motorcontrol.MotorController;
import net.bancino.log.Log;
import net.bancino.log.LogLevel;
import net.bancino.log.Loggable;

/**
 * A virtual motor that implements the WPILib {@code MotorController} interface.
 * This does not map to any hardware so it can be used for testing. Note,
 * however, that this will not change any encoders or gyro angles.
 * 
 * @author Jordan Bancino
 * @version 6.1.0
 * @since 6.1.0
 */
@Loggable
public class VirtualMotor implements MotorController {

    @Log(as = "rawValue", atLevel = LogLevel.IMPORTANT)
    private double speed;

    @Log(atLevel = LogLevel.IMPORTANT)
    private boolean inverted;

    @Override
    public void disable() {
        stopMotor();
    }

    @Override
    public double get() {
        return speed;
    }

    @Override
    public boolean getInverted() {
        return inverted;
    }

    @Override
    public void set(double speed) {
        this.speed = speed;
    }

    @Override
    public void setInverted(boolean isInverted) {
        this.inverted = isInverted;
    }

    @Override
    public void stopMotor() {
        set(0);
    }
}
