/*
 * SwerveIO - A versatile open-source FRC swerve drive library.
 *
 * Copyright (C) 2019-2022 Jordan Bancino <jordan@bancino.net>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published with this
 * source code. You should have received a copy of the GNU General Public
 * License with this program. If you did not, contact the distributors of
 * this program and view it here: https://bancino.net/licenses/gpl-3.0.txt
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 */
/**
 * LogIO: A reflection-based logging library built specifically for SwerveIO.
 *
 * <p>
 * LogIO attempts to solve SwerveIO's logging situation. In previous versions
 * of SwerveIO, an interface had to be implemented to log certain aspects of
 * the SwerveIO runtime. However, this was clunky, because it could only data
 * and objects that were accessible via the public API. This led to a number of
 * variables being exposed via the public API in a way that could allow users to
 * seriously abuse the way SwerveIO works. Furthermore, it made maintaining SwerveIO
 * very difficult because so much of it was exposed to the public API that hardly
 * any of it could be modified as needed in any meaningful way.
 * </p>
 * <p>
 * LogIO solves this situation by using reflection. Although reflection has it's
 * own flaws, it better suits SwerveIO because it allows SwerveIO authors to define
 * how and at what level particular data should be logged without giving SwerveIO
 * users the ability to modify that data in undefined ways. It does this by
 * wrapping all values gathered via reflection in {@link LogEntry}.
 * </p>
 * <p>
 * LogIO is a tree-based logging facility. This means that it recursively processes
 * log targets and generates the log representation in such a way that a log entry may
 * have any number of parent and child enries, depending on where it is in the
 * hierarchy. The generated hierarchy directly mirrors the code from which the
 * hierarchy is derived. LogIO is thus best suited to generating JSON logs that
 * accurately represent the state of a program at a given time, although any
 * situation in which tree-based logging is appropriate can utilize LogIO by implementing
 * {@link LogHandler}.
 * </p>
 *
 * @author Jordan Bancino
 * @version 7.0.0
 * @since 7.0.0
 */
package net.bancino.log;
