/*
 * SwerveIO - A versatile open-source FRC swerve drive library.
 *
 * Copyright (C) 2019-2022 Jordan Bancino <jordan@bancino.net>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published with this
 * source code. You should have received a copy of the GNU General Public
 * License with this program. If you did not, contact the distributors of
 * this program and view it here: https://bancino.net/licenses/gpl-3.0.txt
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 */
/**
 * Encoder iterfaces and helper classes. See {@link Encoder} for details on how
 * an encoder is expected to behave.
 * 
 * <p>
 * Beginning users of SwerveIO will probably want to use an existing encoder
 * implementation, but they can also easily create their own implementations by
 * directly implementing {@link Encoder}.
 * </p>
 * <p>
 * The following encoders are officially supported by SwerveIO Core:
 * </p>
 * <table border="1">
 * <caption style="display: none;">Supported Hardare Table</caption>
 * <tr>
 * <th>Encoder</th>
 * <th>SwerveIO Class</th>
 * </tr>
 * <tr>
 * <td>Any 5-volt analog encoder that plugs into the RoboRIO analog IO port</td>
 * <td>{@link AnalogEncoder}</td>
 * </tr>
 * </table>
 * <p>
 * Additional encoders are supported by the official hardware vendor support
 * libraries. Please refer to the documentation home page for adding hardware
 * support for your encoder.
 * </p>
 * <p>
 * If your encoder is listed as supported, either in the list above or on the
 * home page, use the hardware-specific classes to access them. The
 * documentation for each individual encoder will provide you with all the
 * specifics. If your encoder is not officially supported by SwerveIO, you will
 * have to do a little more work to get SwerveIO to play nice with it, but due
 * to the nature of SwerveIO and the design principles it was built with, adding
 * your own encoder implementation should be trivial.
 * </p>
 * <p>
 * Using the interfaces and classes in this package, you can add support for any
 * encoder.
 * </p>
 * 
 * @author Jordan Bancino
 * @version 6.0.0
 * @since 1.0.0
 */
package net.bancino.robotics.swerveio.encoder;