/*
 * SwerveIO - A versatile open-source FRC swerve drive library.
 *
 * Copyright (C) 2019-2022 Jordan Bancino <jordan@bancino.net>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published with this
 * source code. You should have received a copy of the GNU General Public
 * License with this program. If you did not, contact the distributors of
 * this program and view it here: https://bancino.net/licenses/gpl-3.0.txt
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 */
/**
 * Length, Dimensions, and Vectors. The geometry package provides a number of classes
 * that represent various geometric properties required in driving a swerve drive.
 * The classes in this package are use throughout SwerveIO and are often dependent
 * on one another, so it is important to understand them.
 * 
 * <p>
 * This package does <b>not</b> offer any advanced computations, it merely deals with
 * representing geometric and physical structures in a way that is logical and cohesive
 * for SwerveIO. For details on how the physics and math of SwerveIO works, see the
 * {@link net.bancino.robotics.swerveio.kinematics} package, which details the exact
 * implementations used.
 * </p>
 * 
 * @author Jordan Bancino
 * @version 5.0.0
 * @since 2.0.0
 */
package net.bancino.robotics.swerveio.geometry;