/*
 * SwerveIO - A versatile open-source FRC swerve drive library.
 *
 * Copyright (C) 2019-2022 Jordan Bancino <jordan@bancino.net>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published with this
 * source code. You should have received a copy of the GNU General Public
 * License with this program. If you did not, contact the distributors of
 * this program and view it here: https://bancino.net/licenses/gpl-3.0.txt
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 */
/**
 * Kinematics providers implement the theoretical physics behind the Swerve Drive.
 * It is the job of {@link net.bancino.robotics.swerveio.SwerveDrive} to interpret
 * the vectors produced by the classes in this package, and it is not the job of
 * the classes in this package to determin how the vectors it generates should
 * be achieved. 
 * 
 * <p>
 * The term "kinematics", as used throughout this package's documentation, actually
 * refers to "inverse kinematics". While kinematics takes individual vectors and 
 * generates a vector for the whole, the classes in this package do the opposite:
 * they take a swerve drive vector
 * ({@link net.bancino.robotics.swerveio.geometry.SwerveVector}) and convert it
 * into individual module speeds and angles.
 * </p>
 * 
 * <p>
 * Kinematics providers can have any level of complexity, and can be defined to
 * drive a SwerveIO swerve drive in any number of ways. Each implementation should
 * be thoroughly documented in it's own class documentation. Implementations can
 * be fully "crab mode", or can theoretically drive the swerve drive like a
 * tank-drive.
 * <p>
 * 
 * @author Jordan Bancino
 * @version 2.1.0
 * @since 2.1.0
 */
package net.bancino.robotics.swerveio.kinematics;