/*
 * SwerveIO - A versatile open-source FRC swerve drive library.
 *
 * Copyright (C) 2019-2022 Jordan Bancino <jordan@bancino.net>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published with this
 * source code. You should have received a copy of the GNU General Public
 * License with this program. If you did not, contact the distributors of
 * this program and view it here: https://bancino.net/licenses/gpl-3.0.txt
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 */
/**
 * Swerve Module interfaces and helper classes. See {@link SwerveModule} for
 * details on how a swerve module is expected to behave.
 * 
 * <p>
 * Beginning users of SwerveIO will want to use an existing swerve module
 * implementation, or create their own modules with
 * {@link GenericSwerveModule.Builder}, which creates
 * {@link GenericSwerveModule} instances that implement {@link SwerveModule},
 * allowing them to be used in the construction of a swerve drive. For details
 * on how to construct a swerve drive, see the
 * {@link net.bancino.robotics.swerveio} package overview.
 * </p>
 * <p>
 * This package does not provide any module hardware support itself. Please
 * refer to the documentation home page for adding hardware support for your kit
 * module.
 * </p>
 * <p>
 * If your swerve module is listed as supported, use the hardware-specific
 * classes to implement your swerve modules. The documentation for each
 * individual module class will provide you with all the specifics. If your
 * module is not officially supported by SwerveIO, you will have to do a little
 * bit more work to get SwerveIO to play nice with it, but due to the nature of
 * SwerveIO and the design principles it was build with, adding your own module
 * implementation should be trivial.
 * </p>
 * <p>
 * Using the interfaces and classes in this package, you can add support for
 * <i>any</i> swerve module, be it a kit we haven't heard of, or a custom-built
 * module. If you go this route, you would be most benefited by extending the
 * {@link GenericSwerveModule} class, passing a builder object into the
 * superclass constructor using what was provided by your constructor.
 * {@link GenericSwerveModule} is a fully-supported implementation of
 * {@link SwerveModule}, and the base on which the officially supported modules
 * are built, so you can be sure it will work reliably.
 * </p>
 * <p>
 * {@link GenericSwerveModule} may have some limitations that make it unsuitable
 * for your purposes. If, after trying it, you've decided that this is the case,
 * you are encouraged to directly implement {@link SwerveModule}. This will
 * require most of the boilerplate code that {@link GenericSwerveModule} was
 * intended to eliminate, but it certainly provides much greater control over
 * your module.
 * </p>
 * 
 * @author Jordan Bancino
 * @version 6.0.0
 * @since 1.0.0
 */
package net.bancino.robotics.swerveio.module;