/*
 * SwerveIO - A versatile open-source FRC swerve drive library.
 *
 * Copyright (C) 2019-2022 Jordan Bancino <jordan@bancino.net>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published with this
 * source code. You should have received a copy of the GNU General Public
 * License with this program. If you did not, contact the distributors of
 * this program and view it here: https://bancino.net/licenses/gpl-3.0.txt
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 */
/**
 * The base SwerveIO package that contains the core functionality of SwerveIO.
 * <p>
 * The classes in the package make up the entry point of the SwerveIO API. They
 * bind together all the components present in the subpackages and present an
 * abstract representation of a swerve drive. There are absolutely zero hardware
 * dependencies at this level; everything is abstract.
 * </p>
 * <p>
 * The SwerveDrive class is the programmer's first contact with SwerveIO. It
 * provides a builder for constructing a new swerve drive object capable of 
 * driving any hardware configuration without modification.
 * </p>
 * <p>
 * This package also contains all the enumerations used in this library, which
 * are used primarily for setting up maps and custom configurations.
 * </p>
 * <p>
 * If you are new to SwerveIO and are looking for documentation on setting up
 * your first swerve drive, head over to the {@code SwerveDrive.Builder} class.
 * </p>
 * 
 * @author Jordan Bancino
 * @version 5.0.0
 * @since 1.0.0
 * @see SwerveDrive.Builder
 */
package net.bancino.robotics.swerveio;
