/*
 * SwerveIO - A versatile open-source FRC swerve drive library.
 *
 * Copyright (C) 2019-2022 Jordan Bancino <jordan@bancino.net>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published with this
 * source code. You should have received a copy of the GNU General Public
 * License with this program. If you did not, contact the distributors of
 * this program and view it here: https://bancino.net/licenses/gpl-3.0.txt
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 */
package net.bancino.robotics.swerveio.geometry;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

import org.junit.jupiter.api.Test;

import net.bancino.robotics.swerveio.geometry.Length.Unit;

/**
 * Test the chassis dimension class.
 * 
 * @author Jordan Bancino
 * @version 6.1.0
 * @since 6.1.0
 */
public class ChassisDimensionTest {

    private ChassisDimension testDimension1 = new ChassisDimension(new Length(25, Unit.INCHES));
    private ChassisDimension testDimension2 = new ChassisDimension(new Length(25, Unit.INCHES), new Length(25, Unit.INCHES));
    private ChassisDimension testDimension3 = new ChassisDimension(new Length(0.5, Unit.METERS));

    /**
     * Test {@code equals()}.
     */
    @Test
    public void testEquals() {
        assertEquals(testDimension1, testDimension2);
        assertEquals(testDimension3, new ChassisDimension(new Length(50, Unit.CENTIMETERS)));
    }

    /**
     * Test that the constructor thows errors on invalid input.
     */
    @Test
    public void testConstructor() {
        assertThrows(IllegalArgumentException.class, () -> new ChassisDimension(null));
    }

    /**
     * The the geometry functions.
     */
    @Test
    public void testGeometry() {
        assertEquals(testDimension1.getRadius().get(Unit.BASE_UNIT), testDimension1.getDiagonal().get(Unit.BASE_UNIT) / 2);
        assertEquals(Math.hypot(25, 25), testDimension1.getDiagonal().get(Unit.INCHES));
        assertEquals(Math.hypot(0.5, 0.5) / 2.0, testDimension3.getRadius().get(Unit.METERS));
    }
    
}
