/*
 * SwerveIO - A versatile open-source FRC swerve drive library.
 *
 * Copyright (C) 2019-2022 Jordan Bancino <jordan@bancino.net>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published with this
 * source code. You should have received a copy of the GNU General Public
 * License with this program. If you did not, contact the distributors of
 * this program and view it here: https://bancino.net/licenses/gpl-3.0.txt
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 */
package net.bancino.robotics.swerveio.geometry;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotEquals;

import org.junit.jupiter.api.Test;

/**
 * Test the swerve vector class.
 * 
 * @author Jordan Bancino
 * @version 6.1.0
 * @since 6.1.0
 */
public class SwerveVectorTest {

    private SwerveVector testVector1 = new SwerveVector(0, 0, 0);
    private SwerveVector testVector2 = new SwerveVector(0.25, 0.25, 0.25);
    private SwerveVector testVector3 = new SwerveVector(0.5, 0.5, 0.5);
    private SwerveVector testVector4 = new SwerveVector(2, 2, 2);

    /**
     * Test the math operators ({@code plus()}, {@code minus()}, {@code times()},
     * {@code dividedBy()}).
     */
    @Test
    public void testMath() {
        assertEquals(testVector3, testVector2.plus(testVector2));
        assertEquals(testVector2, testVector3.minus(testVector2));
        assertEquals(testVector3, testVector2.times(testVector4));
        assertEquals(testVector2, testVector3.dividedBy(testVector4));
    }

    /**
     * Test {@code equals()}.
     */
    @Test
    public void testEquals() {
        assertEquals(testVector1, testVector1);
        assertEquals(testVector1, new SwerveVector(0, 0, 0));

        assertNotEquals(testVector1, testVector2);
        assertNotEquals(testVector2, testVector1);
        assertNotEquals(testVector3, testVector4);
    }
}
